/**
* This serves as an example how to make an indicator that has its
* values calculated by a DLL. The code below is only a thin wrapper
* around the DLL, there are no loops, no calculations in mql4,
* it is simply initially setting up the indicator buffer and then
* simply passing every incomiung tick to the DLL which will then
* do all the rest and calculate and write the values to the buffer.
*/
#property copyright "Copyright  2010, Bernd Kreuss"
#property link      "http://sites.google.com/site/prof7bit/"

#property indicator_chart_window
#property indicator_buffers 1
#property indicator_color1 Red

#import "sma.dll"
   // history and buffer are passed by reference, this means the dll will receive pointers to the arrays 
   void updateBuffer(double &history[][6], double &buffer[], int bars, int indicator_counted, int period);
#import

extern int ma_period=13;
extern int ma_shift=0;

double buffer[]; // the indicator buffer
double history[][6]; // this will later point to the complete chart history

int init(){
   // set up the indicator buffer
   SetIndexStyle(0, DRAW_LINE);
   SetIndexShift(0, ma_shift);
   SetIndexBuffer(0, buffer);
   SetIndexLabel(0, "simple moving average");
   
   // the following will initiate some magic from MT4: After 
   // this call the array will automatically be updated by MT4 on
   // every tick. It will from now on behave like the Close[] 
   // or Open[] arrays, only here we have all OHLCV data 
   // conveniently in one big 2-dimensional array.
   // 
   // ArrayCopyRates() does not really copy something, intead it
   // will simply internally change a pointer to make it point
   // to the automatically updating data. Maybe a better name
   // for what really happens here would be GetRatesDataPointer()
   // to retrieve the address where MT4 holds the history but this 
   // would probably only confuse n00bs, since there is no concept
   // of pointers in mql4, although they of course do exist behind
   // the scenes.  
   ArrayCopyRates(history, NULL, 0);
}

/**
* You probably know the standard example from MetaQuotes where 
* they use the value of IndicatorCounted() and the value of Bars 
* and set up a loop that is intended to loop over all new bars 
* (bars that have been newly appeared since start() was called the 
* last time) and then inside this loop they will calculate the 
* indicator value for these bars and write them into the buffer.
*
* All this is now done inside the DLL, and it is done more or 
* less the very same way.
*
* We supply the updateBuffer() function with all data that it 
* needs to do this. It will know the number of bars in the chart,
* the number of already calculated bars and it has a pointer 
* to the buffer array to directly write the values to the buffer. 
* We don't need to touch the buffer from mql4.
*/
int start(){
   updateBuffer(history, buffer, Bars, IndicatorCounted(), ma_period);
}

